@@ -19,7 +19,7 @@ use Encode qw(encode decode);
 use Carp qw(carp croak );
 
 use vars qw'$VERSION %link_spec';
-$VERSION = '0.42';
+$VERSION = '0.43';
 
 =head1 NAME
 
@@ -629,18 +629,37 @@ Retrieves the URL C<URL> into the tab.
 It returns a faked L<HTTP::Response> object for interface compatibility
 with L<WWW::Mechanize>.
 
+Recognized options:
+
+=over 4
+
+=item *
+
+C<< :content_file >> - filename to store the data in
+
+=item *
+
+C<< no_cache >> - if true, bypass the browser cache
+
+=back
+
 =cut
 
 sub get {
     my ($self,$url, %options) = @_;
     my $b = $self->tab->{linkedBrowser};
     $self->clear_current_form;
+    
+    my $flags = 0;
+    if ($options{ no_cache }) {
+        $flags = $self->repl->constant('nsIWebNavigation.LOAD_FLAGS_BYPASS_CACHE');
+    };
 
     if (my $target = delete $options{":content_file"}) {
         $self->save_url($url => $target, %options);
     } else {
         $self->synchronize($self->events, sub {
-            $b->loadURI($url);
+            $b->loadURIWithFlags($url,$flags);
         });
     };
 };
@@ -777,13 +796,10 @@ sub _install_response_header_listener {
     my ($self) = @_;
     
     weaken $self;
-    
-    # These should be cached and optimized into one hash query
-    my $STATE_STOP = $self->repl->constant('Components.interfaces.nsIWebProgressListener.STATE_STOP');
-    my $STATE_IS_DOCUMENT = $self->repl->constant('Components.interfaces.nsIWebProgressListener.STATE_IS_DOCUMENT');
-    my $STATE_IS_WINDOW = $self->repl->constant('Components.interfaces.nsIWebProgressListener.STATE_IS_WINDOW');
 
-    my $make_state_change = $self->repl->declare(<<'JS');
+    # Pre-Filter the progress on the JS side of things so we
+    # don't get that much traffic back and forth between Perl and JS
+    my $make_state_change_filter = $self->repl->declare(<<'JS');
         function (cb) {
             const STATE_STOP = Components.interfaces.nsIWebProgressListener.STATE_STOP;
             const STATE_IS_DOCUMENT = Components.interfaces.nsIWebProgressListener.STATE_IS_DOCUMENT;
@@ -791,27 +807,18 @@ sub _install_response_header_listener {
             
             return function (progress,request,flags,status) {
                 if (flags & (STATE_STOP|STATE_IS_DOCUMENT) == (STATE_STOP|STATE_IS_DOCUMENT)) {
-                    cb(status,request);
+                    cb(progress,request,flags,status);
                 }
             }
         }
 JS
 
-    my $response_received; $response_received = sub {
-        my ($status,$request) = @_;
-        if ($self) {
-            if ($status == 0) {
-                $self->{ response } ||= $request;
-            };
-            #$self->repl->remove_callback( $response_received );
-        };
-        delete $self->{response_received}; # remove ourselves
-    };
-    #$self->{response_received} = $response_received; # need to keep it alive
-    my $state_change = $make_state_change->( $response_received );
-
-=for perl
-    my $state_change = sub {
+    # These should be cached and optimized into one hash query
+    my $STATE_STOP = $self->repl->constant('Components.interfaces.nsIWebProgressListener.STATE_STOP');
+    my $STATE_IS_DOCUMENT = $self->repl->constant('Components.interfaces.nsIWebProgressListener.STATE_IS_DOCUMENT');
+    my $STATE_IS_WINDOW = $self->repl->constant('Components.interfaces.nsIWebProgressListener.STATE_IS_WINDOW');
+    
+    my $state_change = $make_state_change_filter->(sub {
         my ($progress,$request,$flags,$status) = @_;
         #warn sprintf "State     : <progress> <request> %08x %08x\n", $flags, $status;
         #warn sprintf "                                 %08x\n", $STATE_STOP | $STATE_IS_DOCUMENT;
@@ -830,13 +837,7 @@ JS
             #    warn sprintf "%08x", $status;
             #};
         };
-    };
-    #my $status_change = sub {
-    #    my ($progress,$request,$status,$msg) = @_;
-    #    warn sprintf "Status     : <progress> <request> %08x %s\n", $status, $msg;
-    #    warn sprintf "                                 %08x\n", $STATE_STOP;
-    #};
-=cut
+    });
 
     my $browser = $self->tab->{linkedBrowser};
 
@@ -909,10 +910,14 @@ sub _extract_response {
     
     my $nsIHttpChannel = $self->repl->constant('Components.interfaces.nsIHttpChannel');
     my $httpChannel = $request->QueryInterface($nsIHttpChannel);
+    #warn $httpChannel->{originalURI}->{asciiSpec};
     
     my @headers;
     my $v = $self->_headerVisitor(sub{push @headers, @_});
+    
+    # If this fails, we're calling it too early :-(
     $httpChannel->visitResponseHeaders($v);
+    
     my $res = HTTP::Response->new(
         $httpChannel->{responseStatus},
         $httpChannel->{responseStatusText},
@@ -943,8 +948,8 @@ sub response {
             # We're cool!
             return HTTP::Response->new( 200, '', ['Content-Encoding','UTF-8'], encode 'UTF-8' => $self->content);
         } else {
-            # make up a response, below
-            my $url = $self->document->{documentURI};
+            # We'll make up a response, below
+            #my $url = $self->document->{documentURI};
             #carp "Making up a response for unknown URL scheme '$scheme' (from '$url')";
         };
     };
@@ -3300,7 +3305,7 @@ Max Maischein C<corion@cpan.org>
 
 =head1 COPYRIGHT (c)
 
-Copyright 2009-2010 by Max Maischein C<corion@cpan.org>.
+Copyright 2009-2011 by Max Maischein C<corion@cpan.org>.
 
 =head1 LICENSE
 
